#!/bin/csh -f

## Set Debug Parameter 
# DEBUG=1; erase all temporary files 
#       0; leave all temporary files 
set DEBUG=1 
 
## Set Pixel Scale (Not Used)
#set PIXSCALE=0.105  

## Parameters for Photometry
# Radius within which detect object to do photometry
set PHOTRADIUS=100
# Radius of annulus to measure sky
set PHOTAN=10
# Width of annulus to measure sky
set PHOTDAN=10

if ( $#argv != 3 ) then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name)"
    exit 1
endif

set FTABLE=$1
set NPOINT=$2
set HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
set OUTIM=$HEADER".fits"

set MLDATATBL=mldata.tbl
set MLCOMBTBL=mlcomb.tbl

set LDATATBL=ldata.tbl
set LDATATMPTBL=ldata.tmp.tbl
set MLDATATMPTBL=mod_ldata.tmp.tbl
set LDATA1TBL=ldata.1.tbl
set LDATA2TBL=ldata.2.tbl
set SKY1TBL=sky1.tbl
set SKY2TBL=sky2.tbl
set MSKY1TBL=msky1.tbl
set MSKY2TBL=msky2.tbl

set PHOTTABLE=RESULTS/phot.$HEADER
set OFFSETTABLE=RESULTS/offset.$HEADER

set NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

set MASKTBL=tmp.mask
set MASKTABLE=RESULTS/mask.$HEADER
set TMPMASK=tmpoffmask

set DITH=20
@ DITH2 = $DITH * 2

set TMPFILELIST=tmpfile.list
if( -f $TMPFILELIST ) then
  \rm $TMPFILELIST
endif

set CLLOG=log/cl`date +%y%m%d%H%M%S`.log
set CLCOM=cl.com

echo $CLCOM >> $TMPFILELIST

if( !(-e log) ) then
  mkdir log
endif
if( -f $CLLOG ) then
  rm $CLLOG
endif

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"

echo "### Create Offset Masks ###"
echo "### Create Offset Masks ###" >> $CLLOG
gawk '{if($3<=10)print($1,$2)}' $MASKTABLE > $TMPMASK.10
gawk '{if($3==20)print($1,$2)}' $MASKTABLE > $TMPMASK.20
gawk '{if($3==30)print($1,$2)}' $MASKTABLE > $TMPMASK.30
gawk '{if($3==40)print($1,$2)}' $MASKTABLE > $TMPMASK.40
gawk '{if($3==50)print($1,$2)}' $MASKTABLE > $TMPMASK.50
gawk '{if($3>=60)print($1,$2)}' $MASKTABLE > $TMPMASK.60

cat << EOF >> $TMPFILELIST
 $TMPMASK.10 
 $TMPMASK.20 
 $TMPMASK.30 
 $TMPMASK.40 
 $TMPMASK.50 
 $TMPMASK.60 
EOF

\rm MASK/$HEADER.??.0??.tbl
./offsettbl $TMPMASK.10 $OFFSETTABLE MASK/$TMPMASK.10
./offsettbl $TMPMASK.20 $OFFSETTABLE MASK/$TMPMASK.20
./offsettbl $TMPMASK.30 $OFFSETTABLE MASK/$TMPMASK.30
./offsettbl $TMPMASK.40 $OFFSETTABLE MASK/$TMPMASK.40
./offsettbl $TMPMASK.50 $OFFSETTABLE MASK/$TMPMASK.50
./offsettbl $TMPMASK.60 $OFFSETTABLE MASK/$TMPMASK.60

cat << EOF >> $TMPFILELIST
 MASK/$TMPMASK.10.???.tbl 
 MASK/$TMPMASK.20.???.tbl 
 MASK/$TMPMASK.30.???.tbl 
 MASK/$TMPMASK.40.???.tbl 
 MASK/$TMPMASK.50.???.tbl 
 MASK/$TMPMASK.60.???.tbl 
EOF

echo "### Erase Objects ###"
echo "### Erase Objects ###" >> $CLLOG

sed 's/sky1/msky1/g' $SKY1TBL > $MSKY1TBL
if( $NFRAME > 1 ) then
  sed 's/sky2/msky2/g' $SKY2TBL > $MSKY2TBL
endif
echo "imdel @$MLDATATBL" > $CLCOM
echo "imdel @$MSKY1TBL " >> $CLCOM
echo "imdel @$MSKY2TBL " >> $CLCOM
echo "imarith @$SKY1TBL - sky1 @$SKY1TBL " >> $CLCOM
echo "imarith @$SKY2TBL - sky2 @$SKY2TBL " >> $CLCOM
echo "logout" >> $CLCOM 
cat $CLCOM | cl >> $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

set i=1
while($i <= $NPOINT)
  echo "# Erasing Objects in Position $i #"
  echo "# Erasing Objects in Position $i #" >> $CLLOG
  set filename1=`sed -n ${i}p $SKY1TBL`
  set filename2=`sed -n ${i}p $MSKY1TBL`
  set POINTNUM=`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`
  if(-f $MASKTBL) then 
    \rm $MASKTBL
  endif
  foreach filename (`ls MASK/$TMPMASK.??.$POINTNUM.tbl`)
   if( !(-z $filename) ) then
   echo $filename >> $MASKTBL
#   echo $filename
   endif
  end
  echo "eraseobj $filename1 $filename2 @$MASKTBL" > $CLCOM
#  echo "eraseobj $filename1 $filename2 MASK/$TMPMASK."`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`".tbl" > $CLCOM
  if( $NFRAME > 1 ) then
    set filename1=`sed -n ${i}p $SKY2TBL`
    set filename2=`sed -n ${i}p $MSKY2TBL`
    echo "eraseobj $filename1 $filename2 @$MASKTBL" >> $CLCOM
#    echo "eraseobj $filename1 $filename2 MASK/$TMPMASK."`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`".tbl" >> $CLCOM
  endif
  echo "logout" >> $CLCOM 
  cat $CLCOM | cl >>& $CLLOG
  if ( $status != 0 ) then
   echo "# Error in IRAF ... Exiting #"
   echo "# Refer to IRAF log $CLLOG. #" 
   exit 1
  endif
  @ i += 1
end
echo $LDATATMPTBL >> $TMPFILELIST
echo $MLDATATMPTBL >> $TMPFILELIST

echo "# Adding Sky Again #"
echo "# Adding Sky Again #" >> $CLLOG
echo "imdel @$SKY1TBL" > $CLCOM
echo "imdel @$SKY2TBL" >> $CLCOM
echo "imarith @$MSKY1TBL + sky1 @$SKY1TBL" >> $CLCOM
if ( $NFRAME > 1 ) then
  echo "imarith @$MSKY2TBL + sky2 @$SKY2TBL" >> $CLCOM
endif
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

echo "### Re-Create Sky Frames ###"
echo "### Re-Create Sky Frames ###" >> $CLLOG
echo "imdel noobjsky1,noobjsky2" > $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif
echo "# Combine Noobj Sky #"
echo "imcombine @$SKY1TBL noobjsky1 combine=median reject=sigclip outtype=real lsigma=2 hsigma=2" > $CLCOM
if ( $NFRAME > 1 ) then 
  echo "imcombine @$SKY2TBL noobjsky2 combine=median reject=sigclip outtype=real lsigma=2 hsigma=2" >> $CLCOM
endif
echo "logout" >> $CLCOM 
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

echo "### Subtract New Sky Frames ###"
echo "### Subtract New Sky Frames ###" >> $CLLOG
echo "imarith @$LDATA1TBL - noobjsky1 @$LDATA1TBL" > $CLCOM
if ( $NFRAME > 1 ) then 
 echo "imarith @$LDATA2TBL - noobjsky2 @$LDATA2TBL" >> $CLCOM
endif
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

echo "### Modify ###"
echo "### Modify ###" >> $CLLOG
echo "imdel @$MLDATATBL" > $CLCOM
echo "logout" >> $CLCOM 
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

set i=1
while($i <= $NPOINT)
echo "# Adjusting Bias Offset in Position $i #"  
echo "# Adjusting Bias Offset in Position $i #" >> $CLLOG
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' $LDATATBL > $LDATATMPTBL
  sed 's/ldata/mldata/g' $LDATATMPTBL > $MLDATATMPTBL
  set POINTNUM=`gawk -v n=$i 'BEGIN{printf("%03d\n",n)}'`

  if(-f $MASKTBL) then 
  \rm $MASKTBL
  endif
  foreach filename (`ls MASK/$TMPMASK.??.$POINTNUM.tbl`)
   if( !(-z $filename) ) then
     echo $filename >> $MASKTBL
   endif
  end
  echo "modify @$LDATATMPTBL @$MLDATATMPTBL @$MASKTBL" > $CLCOM
  echo "logout" >> $CLCOM 
  cat $CLCOM | cl >>& $CLLOG
  if ( $status != 0 ) then
   echo "# Error in IRAF ... Exiting #"
   echo "# Refer to IRAF log $CLLOG. #" 
   exit 1
  endif
  @ i += 1
end

echo "### Shift and Combine All Frames ###"
echo "### Shift and Combine All Frames ###" >> $CLLOG

echo "# Calculate Sigma for Single Frame #"
echo "# Calculate Sigma for Single Frame #" >> $CLLOG
echo "del tmp.sigma" > $CLCOM 
echo "imstatrej `sed -n 1p $MLDATATBL`[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif
set SIGMA=`gawk '{if(NR==1)print($3*1.5)}' tmp.sigma` 
echo "  sigma=$SIGMA"
echo tmp.sigma >> $TMPFILELIST

echo $MLCOMBTBL >> $TMPFILELIST
if ( -f $MLCOMBTBL ) then
  \rm $MLCOMBTBL
endif
set i=1
while($i <= $NPOINT)
echo "# Shift and Add at Position $i #"
echo "# Shift and Add at Position $i #" >> $CLLOG
  echo " # Object Detection #"
  echo " # Object Detection #" >> $CLLOG
  set PMLTABLE=mldata.$i.tbl
  echo $PMLTABLE >> $TMPFILELIST
  set PMLCOMB=RESULTS/$HEADER.$i.fits
  gawk -v i=$i -v nf=$NFRAME '{if(NR>nf*(i-1) && NR<=nf*i) print($0)}' $MLDATATBL > $PMLTABLE
  echo "noao" > $CLCOM 
  echo "digiphot" >> $CLCOM 
  echo "apphot" >> $CLCOM
  foreach filename (`cat $PMLTABLE`) 
    set COORDFILE=`echo $filename | sed 's/.imh/.coo/g' | sed 's/.fits/.coo/g'`
    echo $COORDFILE >> $TMPFILELIST
    if ( -f $COORDFILE ) then
      \rm $COORDFILE 
    endif
    echo "imdel tmpmed" >> $CLCOM
    echo "median "$filename" tmpmed 3 3" >> $CLCOM
    echo "daofind tmpmed output="$COORDFILE" fwhmpsf=8 thresho=4 verify- sigma="$SIGMA" sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> $CLCOM 
    echo "display $filename 1 fi+" >> $CLCOM
    echo "tvmark 1 $COORDFILE number+" >> $CLCOM
  end 
  echo "logout" >> $CLCOM
  cat $CLCOM | cl >>& $CLLOG
  if ( $status != 0 ) then
   echo "# Error in IRAF ... Exiting #"
   echo "# Refer to IRAF log $CLLOG. #" 
   exit 1
  endif
  \cp $COORDFILE tmp.coord
  grep -v "^#" tmp.coord > $COORDFILE
  \rm tmp.coord
  
  echo " # Pattern Matching #"
  echo " # Pattern Matching #" >> $CLLOG
  @ j = 1 
  foreach filename (`cat $PMLTABLE`)  
    set COORDFILE=`echo $filename | sed 's/.imh/.coo/g'| sed 's/.fits/.coo/g'`  
#    echo $COORDFILE $j 
    if ( $j == 1 ) then 
      set BASEFILE=$COORDFILE
      echo "0 0" > tmp.offset 
    else 
      ./matchpat $BASEFILE $COORDFILE 10 >> tmp.offset
    endif 
    @ j = $j + 1
  end
  echo " # Combine frames: Output is $PMLCOMB #"
  echo " # Combine frames: Output is $PMLCOMB #" >> $CLLOG
  echo "imdel $PMLCOMB" > $CLCOM
  echo "imcombine @$PMLTABLE $PMLCOMB combine=average reject=sigclip outtype=real lsigma=2 hsigma=2 offset=tmp.offset" >> $CLCOM
  echo "display $PMLCOMB 1 fi+" >> $CLCOM
  echo "logout" >> $CLCOM
  cat $CLCOM | cl >>& $CLLOG
  if ( $status != 0 ) then
   echo "# Error in IRAF ... Exiting #"
   echo "# Refer to IRAF log $CLLOG. #" 
   exit 1
  endif
  echo $PMLCOMB >> $MLCOMBTBL
  @ i += 1
end
echo tmp.offset >> $TMPFILELIST



echo "### Pattern Matching of Coadded Frames ###"
echo "### Pattern Matching of Coadded Frames ###" >> $CLLOG 
echo "# Calculate Sigma for Single Frame #"
echo "# Calculate Sigma for Single Frame #" >> $CLLOG 
echo "del tmp.sigma" > $CLCOM 
echo "imstatrej `sed -n 1p $MLCOMBTBL`[300:600,300:600] 3 > tmp.sigma" >> $CLCOM
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif
set SIGMA=`gawk '{if(NR==1)print($3*1)}' tmp.sigma` 
echo "  sigma=$SIGMA" 

echo " # Object Detection #" 
echo " # Object Detection #" >> $CLLOG 
echo "noao" > $CLCOM
echo "digiphot" >> $CLCOM
echo "apphot" >> $CLCOM
foreach filename (`cat $MLCOMBTBL`)  
  set COORDFILE=`echo $filename | sed 's/.imh/.coo/g' | sed 's/.fits/.coo/g'`  
  if( -f $COORDFILE ) then
    \rm $COORDFILE 
  endif
  echo "imdel tmpmed" >> $CLCOM
  echo "median "$filename" tmpmed 3 3" >> $CLCOM
  echo "daofind tmpmed output="$COORDFILE" fwhmpsf=8 thresho=4 verify- sigma="$SIGMA" sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> $CLCOM 
  echo "display $filename 1 fi+" >> $CLCOM
  echo "tvmark 1 $COORDFILE number+" >> $CLCOM
end  
echo "logout" >> $CLCOM 
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif

@ i = 1 
foreach filename (`cat $MLCOMBTBL`)  
  set COORDFILE=`echo $filename | sed 's/.imh/.coo/g'| sed 's/.fits/.coo/g'`  
  set COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g' | sed 's/.fits/2.coo/g'`  
  set XOFFSET=`sed -n ${i}p $OFFSETTABLE | gawk '{print($1)}'`
  set YOFFSET=`sed -n ${i}p $OFFSETTABLE | gawk '{print($2)}'`
  grep -v "^#" $COORDFILE | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $COORDFILE2
  echo $COORDFILE >> $TMPFILELIST
  echo $COORDFILE2 >> $TMPFILELIST
  @ i = $i + 1
end

echo "# Pattern Matching #"
echo "# Pattern Matching #" >> $CLLOG
@ i = 1 
foreach filename (`cat $MLCOMBTBL`)   
  set COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g' | sed 's/.fits/2.coo/g'`  
#  echo $COORDFILE2 $i
  if ( $i == 1 ) then
    set BASEFILE=$COORDFILE2
    echo "0 0" > tmp.offset
  else
    ./matchpat $BASEFILE $COORDFILE2 5 >> tmp.offset
  endif
  @ i = $i + 1
end
paste $OFFSETTABLE tmp.offset | gawk '{print($1+$3,$2+$4)}' > tmp.offset2
echo tmp.offset2 >> $TMPFILELIST

echo "### Shift and Add ###"
echo "### Shift and Add ###" >> $CLLOG
echo "imdel tmp.tot.imh" > $CLCOM
echo "imcombine @$MLCOMBTBL tmp.tot combine=median reject=sigclip outtype=real lsigma=2 hsigma=2 offset=tmp.offset2" >> $CLCOM
echo "display tmp.tot 1 fi+" >> $CLCOM
echo "imdel RESULTS/$OUTIM" >> $CLCOM 
echo "imcopy tmp.tot RESULTS/$OUTIM" >> $CLCOM 
echo "logout" >> $CLCOM
cat $CLCOM | cl >>& $CLLOG
if ( $status != 0 ) then
 echo "# Error in IRAF ... Exiting #"
 echo "# Refer to IRAF log $CLLOG. #" 
 exit 1
endif
\cp tmp.offset2 $OFFSETTABLE

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
echo "Final frame is saved as RESULTS/"$HEADER".fits"
echo "Single frames are saved as;"
cat $MLCOMBTBL
echo "Offset pattern file is saved as "$OFFSETTABLE
echo "Offset pattern is : " 
cat $OFFSETTABLE
echo "IRAF log written in $CLLOG"
if ( $DEBUG == 1 ) then
echo "### Erase Temporary Files ###"
echo "### Erase Temporary Files ###" >> $CLLOG
  foreach filename (`cat $TMPFILELIST`)
      \rm $filename
  end
endif
